"""
kzconfig.context
~~~~~

Kazoo config library.


"""

from . import kube


class context:
    env = kube.api.get('configmap', 'environment').data
    couch = kube.api.get('secret', 'couchdb').data
    rabbit = kube.api.get('secret', 'couchdb').data
    master_acct = kube.api.get('secret', 'master-account').data
    dns = kube.api.get('secret', 'dns.dnsimple').data



"""
kzconfig.cli
~~~~~

Kazoo config library.

A sup cli command for invoking sup against a remote kazoo container.
"""

import click

from ..sup import sup, sup_api


@click.command()
@click.option('--kube', 'method', flag_value='kube', default=True)
@click.option('--api', 'method', flag_value='api')
@click.argument('module')
@click.argument('function')
@click.argument('args', nargs=-1)
def sup_cmd(method, module, function, args):
    if method == 'api':
        func = sup_api
    else:
        func = sup
    return click.echo(func(module, function, *args))



from dnsimple import DNSimple

from .context import context

api = DNSimple(
    username=context.dns['email'],
    password=context.dns['password']
)


def add(kind='A', name='', content='', **kwargs):
    data = dict(
        record_type=kind.upper(),
        name=name,
        content=content
    )
    data.update(**kwargs)
    api.add_record(context.env['company.domain'], data)



"""
kzconfig.kube
~~~~~

Kazoo config library.


"""

import pyrkube


api = pyrkube.KubeAPIClient()


def get_pod(name):
    pods = api.get('pod', selector=dict(app=name))
    if pods:
        pod = pods[0]
        return pod.name



"""
kzconfig.kazoo
~~~~~

Kazoo config library.


"""

import kazoo

from .context import context


master_acct = context.master_acct
env = context.env

# print(master_acct['user'], master_acct['pass'], master_acct['name'], env['uri.crossbar'])

api = kazoo.Client(
    username=master_acct['user'],
    password=master_acct['pass'],
    account_name=master_acct['name'],
    base_url=env['uri.crossbar']
)

api.authenticate()



"""
kzconfig.couch
~~~~~

Kazoo config library.


"""

from urllib.parse import urlparse, unquote
import json
import couchdb

from . import util
from .context import context


p = urlparse(context.env['uri.couchdb'])
api = couchdb.Server(util.join_url(
    p.scheme, context.couch['user'], context.couch['pass'], p.netloc
))


def create_doc(db, doc):
    if isinstance(doc, str):
        doc = json.loads(doc)
    doc_id = doc['_id']

    db = api[db]
    if doc_id in db:
        old_doc = db[doc_id]
        doc['_rev'] = old_doc.rev
    return db.save(doc)


def get_db_for(acct_id):
    db = api['accounts']
    doc = db[acct_id]
    db_name = doc['pvt_account_db']
    print(db_name)
    return api[unquote(db_name)]



"""
kzconfig.sup
~~~~~

Kazoo config library.


"""

import subprocess
from decimal import Decimal

from . import kazoo, kube, util


class KappsMaint:
    module = 'kapps_maintenance'

    @staticmethod
    def refresh(database=''):
        return sup(KappsMaint.module, 'refresh', database)


class KappsAcctConfig:
    module = 'kapps_account_config'

    @staticmethod
    def get(acct, doc, key, default='undefined'):
        if not isinstance(default, str):
            default = str(default)
        return sup(KappsAcctConfig.module, 'get', acct, doc, key, default)

    @staticmethod
    def set(acct, doc, key, value):
        if isinstance(value, bytes):
            value = value.decode()
        return sup(KappsAcctConfig.module, 'set', acct, doc, key, value)

    @staticmethod
    def flush(acct, doc=None, strategy=None):
        args = [acct, doc]
        if strategy:
            args.append(strategy)
        return sup(KappsAcctConfig.module, 'flush', *args)


class KappsConfig:
    module = 'kapps_config'

    @staticmethod
    def get(doc, key, default='undefined', node=None):
        if not isinstance(default, str):
            default = str(default)

        args = [doc, key, default]
        if node:
            args.append(node)
        return sup(KappsConfig.module, 'get', *args)

    @staticmethod
    def set(doc, key, value, node=None):
        if isinstance(value, bytes):
            value = value.decode()

        if isinstance(value, bool):
            func = 'set_boolean'
            value = str(value).lower()
        elif isinstance(value, int):
            func = 'set_integer'
        elif isinstance(value, (float, Decimal)):
            func = 'set_float'
            value = str(value)
        else:
            func = 'set'

        args = [doc, key, value]
        if node:
            args.append(node)
        return sup(KappsConfig.module, func, *args)

    @staticmethod
    def set_json(doc, key, value):
        if not isinstance(value, str):
            value = util.json_dumps(value)

        return sup(KappsConfig.module, 'set_json', doc, key, value)

    @staticmethod
    def set_default(doc, key, value):
        return sup(KappsConfig.module, 'set_default', doc, key, value)

    @staticmethod
    def flush(doc=None, key=None, node=None):
        args = []
        for arg in (doc, key, node):
            if arg:
                args.append(arg)

        return sup(KappsConfig.module, 'flush', *args)


def sup(module, function, *args):
    args = list(args)
    pod = kube.get_pod('kazoo')
    for idx, arg in enumerate(args):
        if isinstance(arg, (int, bool)):
            args[idx] = str(arg)

    args_str = ' '.join([module, function, *args])
    cmd = 'kubectl exec {} -- sup {}'.format(pod, args_str)
    print(cmd)
    return subprocess.getoutput(cmd)


def sup_api(module, function, *args):
    _, result = kazoo.api.sup(module, function, *args)
    return result


def status():
    return sup('kz_nodes', 'status')


# def db_refresh(database=''):
#     return sup('kapps_maintenance', 'refresh', database)


# def acct_flush(acct_id):
#     return sup('kapps_account_config', 'flush', acct_id)


# for backwards compatibility
# def config_set_json(*args, **kwargs):
#     return KappsConfig.set_json(*args, **kwargs)
#
#
# def config_flush(*args, **kwargs):
#     return KappsConfig.flush(*args, **kwargs)
#
#
# def config_set_default(*args, **kwargs):
#     return KappsConfig.set_default(*args, **kwargs)
